package java.security;

import sun.security.util.Debug;
import sun.reflect.Reflection;
import sun.reflect.CallerSensitive;

public final class AccessController
{
    private AccessController() {
    }
    
    @CallerSensitive
    public static native <T> T doPrivileged(PrivilegedAction<T> p0);
    
    @CallerSensitive
    public static <T> T doPrivilegedWithCombiner(PrivilegedAction<T> privilegedAction) {
        AccessControlContext stackAccessControlContext = getStackAccessControlContext();
        if (stackAccessControlContext == null) {
            return doPrivileged(privilegedAction);
        }
        return doPrivileged(privilegedAction, preserveCombiner(stackAccessControlContext.getAssignedCombiner(), Reflection.getCallerClass()));
    }
    
    @CallerSensitive
    public static native <T> T doPrivileged(PrivilegedAction<T> p0, AccessControlContext p1);
    
    @CallerSensitive
    public static <T> T doPrivileged(PrivilegedAction<T> privilegedAction, AccessControlContext accessControlContext, Permission... array) {
        AccessControlContext context = getContext();
        if (array == null) {
            throw new NullPointerException("null permissions parameter");
        }
        return doPrivileged(privilegedAction, createWrapper((accessControlContext == null) ? null : accessControlContext.getCombiner(), Reflection.getCallerClass(), context, accessControlContext, array));
    }
    
    @CallerSensitive
    public static <T> T doPrivilegedWithCombiner(PrivilegedAction<T> privilegedAction, AccessControlContext accessControlContext, Permission... array) {
        AccessControlContext context = getContext();
        DomainCombiner domainCombiner = context.getCombiner();
        if (domainCombiner == null && accessControlContext != null) {
            domainCombiner = accessControlContext.getCombiner();
        }
        if (array == null) {
            throw new NullPointerException("null permissions parameter");
        }
        return doPrivileged(privilegedAction, createWrapper(domainCombiner, Reflection.getCallerClass(), context, accessControlContext, array));
    }
    
    @CallerSensitive
    public static native <T> T doPrivileged(PrivilegedExceptionAction<T> p0) throws PrivilegedActionException;
    
    @CallerSensitive
    public static <T> T doPrivilegedWithCombiner(PrivilegedExceptionAction<T> privilegedExceptionAction) throws PrivilegedActionException {
        AccessControlContext stackAccessControlContext = getStackAccessControlContext();
        if (stackAccessControlContext == null) {
            return doPrivileged(privilegedExceptionAction);
        }
        return doPrivileged(privilegedExceptionAction, preserveCombiner(stackAccessControlContext.getAssignedCombiner(), Reflection.getCallerClass()));
    }
    
    private static AccessControlContext preserveCombiner(DomainCombiner domainCombiner, Class<?> clazz) {
        return createWrapper(domainCombiner, clazz, null, null, (Permission[])null);
    }
    
    private static AccessControlContext createWrapper(DomainCombiner domainCombiner, Class<?> clazz, AccessControlContext accessControlContext, AccessControlContext accessControlContext2, Permission[] array) {
        ProtectionDomain callerPD = getCallerPD(clazz);
        if (accessControlContext2 != null && !accessControlContext2.isAuthorized() && System.getSecurityManager() != null && !callerPD.impliesCreateAccessControlContext()) {
            return new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, null) });
        }
        return new AccessControlContext(callerPD, domainCombiner, accessControlContext, accessControlContext2, array);
    }
    
    private static ProtectionDomain getCallerPD(final Class<?> clazz) {
        return (ProtectionDomain)doPrivileged(new PrivilegedAction<ProtectionDomain>() {
            @Override
            public ProtectionDomain run() {
                return clazz.getProtectionDomain();
            }
        });
    }
    
    @CallerSensitive
    public static native <T> T doPrivileged(PrivilegedExceptionAction<T> p0, AccessControlContext p1) throws PrivilegedActionException;
    
    @CallerSensitive
    public static <T> T doPrivileged(PrivilegedExceptionAction<T> privilegedExceptionAction, AccessControlContext accessControlContext, Permission... array) throws PrivilegedActionException {
        AccessControlContext context = getContext();
        if (array == null) {
            throw new NullPointerException("null permissions parameter");
        }
        return doPrivileged(privilegedExceptionAction, createWrapper((accessControlContext == null) ? null : accessControlContext.getCombiner(), Reflection.getCallerClass(), context, accessControlContext, array));
    }
    
    @CallerSensitive
    public static <T> T doPrivilegedWithCombiner(PrivilegedExceptionAction<T> privilegedExceptionAction, AccessControlContext accessControlContext, Permission... array) throws PrivilegedActionException {
        AccessControlContext context = getContext();
        DomainCombiner domainCombiner = context.getCombiner();
        if (domainCombiner == null && accessControlContext != null) {
            domainCombiner = accessControlContext.getCombiner();
        }
        if (array == null) {
            throw new NullPointerException("null permissions parameter");
        }
        return doPrivileged(privilegedExceptionAction, createWrapper(domainCombiner, Reflection.getCallerClass(), context, accessControlContext, array));
    }
    
    private static native AccessControlContext getStackAccessControlContext();
    
    static native AccessControlContext getInheritedAccessControlContext();
    
    public static AccessControlContext getContext() {
        AccessControlContext stackAccessControlContext = getStackAccessControlContext();
        if (stackAccessControlContext == null) {
            return new AccessControlContext(null, true);
        }
        return stackAccessControlContext.optimize();
    }
    
    public static void checkPermission(Permission obj) throws AccessControlException {
        if (obj == null) {
            throw new NullPointerException("permission can't be null");
        }
        AccessControlContext stackAccessControlContext = getStackAccessControlContext();
        if (stackAccessControlContext == null) {
            Debug debug = AccessControlContext.getDebug();
            boolean n = false;
            if (debug != null) {
                n = (!Debug.isOn("codebase=") & (!Debug.isOn("permission=") || Debug.isOn("permission=" + obj.getClass().getCanonicalName())));
            }
            if (n && Debug.isOn("stack")) {
                Thread.dumpStack();
            }
            if (n && Debug.isOn("domain")) {
                debug.println("domain (context is null)");
            }
            if (n != 0) {
                debug.println("access allowed " + obj);
            }
            return;
        }
        else {
            stackAccessControlContext.optimize().checkPermission(obj);
            return;
        }
    }
}
